package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.numberrule;

import cn.estsh.i3plus.ext.mes.apiservice.utils.MesException;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.mes.api.iservice.busi.INumberRulePackAttributeStrategyService;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesPart;
import cn.estsh.i3plus.pojo.mes.model.GenSerialNoModel;
import cn.estsh.i3plus.pojo.mes.repository.MesPartRepository;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import java.util.Date;

/**
 * 重庆龙兴客户条码
 */
@Component
public class CqLxCustSnNumberRuleStrategyService implements INumberRulePackAttributeStrategyService {

    @Autowired
    private MesPartRepository partRepository;

    @Override
    public GenSerialNoModel execute(GenSerialNoModel genSerialNoModel) {
        if (null == genSerialNoModel) {
            MesException.throwBusiException("入参缺少[GenSerialNoModel]!");
        }
        if (StringUtils.isEmpty(genSerialNoModel.getOrganizeCode())) {
            MesException.throwBusiException("入参缺少[organizeCode]!");
        }

        MesPart part = (null != genSerialNoModel.getDataMap() && genSerialNoModel.getDataMap().containsKey(MesPart.class.getSimpleName()))
                ? (MesPart) genSerialNoModel.getDataMap().get(MesPart.class.getSimpleName())
                : getMesPart(genSerialNoModel.getOrganizeCode(), genSerialNoModel.getPartNo());

        if (null == part) {
            MesException.throwBusiException("入参缺少[part]!");
        }

        if (StringUtils.isEmpty(part.getPartSnParam())) {
            MesException.throwBusiException("零件编码[%s]未维护零件条码参数!", part.getPartNo());
        }

        genSerialNoModel.setPartSnParam(part.getPartSnParam());
        //年月日缩写
        Date date = new Date();
        genSerialNoModel.setYear(getYearShort(date));
        genSerialNoModel.setMonth(getMonthShort(date));
        genSerialNoModel.setDay(getDayShort(date));
        return genSerialNoModel;
    }

    private String getYearShort(Date date) {
        return MesExtEnumUtil.YEAR_SHORT.valueOfDescription(Integer.parseInt(TimeTool.getYear(date)));
    }

    private String getMonthShort(Date date) {
        return MesExtEnumUtil.MONTH_SHORT.valueOfDescription(Integer.parseInt(TimeTool.getMonth(date)));
    }

    private String getDayShort(Date date) {
        return MesExtEnumUtil.DAY_SHORT.valueOfDescription(Integer.parseInt(TimeTool.getDay(date)));
    }

    private MesPart getMesPart(String orgaizeCode, String partNo) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(orgaizeCode);
        DdlPreparedPack.getStringEqualPack(partNo, MesExtConstWords.PART_NO, ddlPackBean);
        return partRepository.getByProperty(ddlPackBean);
    }

}